Method: Blazer::SlackNotifier.query_url

Defined in:
lib/blazer/slack_notifier.rb

.query_url(id) ⇒ Object

checks shouldn’t have variables, but in any case, avoid passing variable params to url helpers (known unsafe parameters are removed, but still not ideal)



66
67
68
# File 'lib/blazer/slack_notifier.rb', line 66

def self.query_url(id)
  Blazer::Engine.routes.url_helpers.query_url(id, ActionMailer::Base.default_url_options)
end